@@ -1,8 +1,9 @@
-/* A Bison parser, made by GNU Bison 3.0.5.  */
+/* A Bison parser, made by GNU Bison 3.3.2.  */
 
 /* Bison implementation for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015, 2018 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2019 Free Software Foundation,
+   Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -40,11 +41,14 @@
    define necessary library symbols; they are noted "INFRINGES ON
    USER NAME SPACE" below.  */
 
+/* Undocumented macros, especially those whose name start with YY_,
+   are private implementation details.  Do not rely on them.  */
+
 /* Identify Bison output.  */
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "3.0.5"
+#define YYBISON_VERSION "3.3.2"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -69,8 +73,8 @@
 #define yylval          i386_lval
 #define yychar          i386_char
 
-/* Copy the first part of user declarations.  */
-#line 1 "i386_parse.y" /* yacc.c:339  */
+/* First part of user prologue.  */
+#line 1 "i386_parse.y" /* yacc.c:337  */
 
 /* Parser for i386 CPU description.
    Copyright (C) 2004, 2005, 2007, 2008, 2009 Red Hat, Inc.
@@ -285,13 +289,16 @@ extern FILE *outfile;
 static size_t best_mnemonic_bits;
 #endif
 
-#line 289 "i386_parse.c" /* yacc.c:339  */
-
+#line 293 "i386_parse.c" /* yacc.c:337  */
 # ifndef YY_NULLPTR
-#  if defined __cplusplus && 201103L <= __cplusplus
-#   define YY_NULLPTR nullptr
+#  if defined __cplusplus
+#   if 201103L <= __cplusplus
+#    define YY_NULLPTR nullptr
+#   else
+#    define YY_NULLPTR 0
+#   endif
 #  else
-#   define YY_NULLPTR 0
+#   define YY_NULLPTR ((void*)0)
 #  endif
 # endif
 
@@ -349,7 +356,7 @@ extern int i386_debug;
 
 union YYSTYPE
 {
-#line 216 "i386_parse.y" /* yacc.c:355  */
+#line 216 "i386_parse.y" /* yacc.c:352  */
 
   unsigned long int num;
   char *str;
@@ -359,7 +366,7 @@ union YYSTYPE
   struct argname *name;
   struct argument *arg;
 
-#line 363 "i386_parse.c" /* yacc.c:355  */
+#line 370 "i386_parse.c" /* yacc.c:352  */
 };
 
 typedef union YYSTYPE YYSTYPE;
@@ -374,9 +381,7 @@ int i386_parse (void);
 
 #endif /* !YY_I386_I_PARSE_H_INCLUDED  */
 
-/* Copy the second part of user declarations.  */
 
-#line 380 "i386_parse.c" /* yacc.c:358  */
 
 #ifdef short
 # undef short
@@ -397,13 +402,13 @@ typedef signed char yytype_int8;
 #ifdef YYTYPE_UINT16
 typedef YYTYPE_UINT16 yytype_uint16;
 #else
-typedef unsigned short int yytype_uint16;
+typedef unsigned short yytype_uint16;
 #endif
 
 #ifdef YYTYPE_INT16
 typedef YYTYPE_INT16 yytype_int16;
 #else
-typedef short int yytype_int16;
+typedef short yytype_int16;
 #endif
 
 #ifndef YYSIZE_T
@@ -415,7 +420,7 @@ typedef short int yytype_int16;
 #  include <stddef.h> /* INFRINGES ON USER NAME SPACE */
 #  define YYSIZE_T size_t
 # else
-#  define YYSIZE_T unsigned int
+#  define YYSIZE_T unsigned
 # endif
 #endif
 
@@ -451,15 +456,6 @@ typedef short int yytype_int16;
 # define YY_ATTRIBUTE_UNUSED YY_ATTRIBUTE ((__unused__))
 #endif
 
-#if !defined _Noreturn \
-     && (!defined __STDC_VERSION__ || __STDC_VERSION__ < 201112)
-# if defined _MSC_VER && 1200 <= _MSC_VER
-#  define _Noreturn __declspec (noreturn)
-# else
-#  define _Noreturn YY_ATTRIBUTE ((__noreturn__))
-# endif
-#endif
-
 /* Suppress unused-variable warnings by "using" E.  */
 #if ! defined lint || defined __GNUC__
 # define YYUSE(E) ((void) (E))
@@ -467,7 +463,7 @@ typedef short int yytype_int16;
 # define YYUSE(E) /* empty */
 #endif
 
-#if defined __GNUC__ && 407 <= __GNUC__ * 100 + __GNUC_MINOR__
+#if defined __GNUC__ && ! defined __ICC && 407 <= __GNUC__ * 100 + __GNUC_MINOR__
 /* Suppress an incorrect diagnostic about yylval being uninitialized.  */
 # define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN \
     _Pragma ("GCC diagnostic push") \
@@ -629,16 +625,16 @@ union yyalloc
 /* YYNSTATES -- Number of states.  */
 #define YYNSTATES  49
 
-/* YYTRANSLATE[YYX] -- Symbol number corresponding to YYX as returned
-   by yylex, with out-of-bounds checking.  */
 #define YYUNDEFTOK  2
 #define YYMAXUTOK   267
 
+/* YYTRANSLATE(TOKEN-NUM) -- Symbol number corresponding to TOKEN-NUM
+   as returned by yylex, with out-of-bounds checking.  */
 #define YYTRANSLATE(YYX)                                                \
-  ((unsigned int) (YYX) <= YYMAXUTOK ? yytranslate[YYX] : YYUNDEFTOK)
+  ((unsigned) (YYX) <= YYMAXUTOK ? yytranslate[YYX] : YYUNDEFTOK)
 
 /* YYTRANSLATE[TOKEN-NUM] -- Symbol number corresponding to TOKEN-NUM
-   as returned by yylex, without out-of-bounds checking.  */
+   as returned by yylex.  */
 static const yytype_uint8 yytranslate[] =
 {
        0,     2,     2,     2,     2,     2,     2,     2,     2,     2,
@@ -812,22 +808,22 @@ static const yytype_uint8 yyr2[] =
 
 #define YYRECOVERING()  (!!yyerrstatus)
 
-#define YYBACKUP(Token, Value)                                  \
-do                                                              \
-  if (yychar == YYEMPTY)                                        \
-    {                                                           \
-      yychar = (Token);                                         \
-      yylval = (Value);                                         \
-      YYPOPSTACK (yylen);                                       \
-      yystate = *yyssp;                                         \
-      goto yybackup;                                            \
-    }                                                           \
-  else                                                          \
-    {                                                           \
-      yyerror (YY_("syntax error: cannot back up")); \
-      YYERROR;                                                  \
-    }                                                           \
-while (0)
+#define YYBACKUP(Token, Value)                                    \
+  do                                                              \
+    if (yychar == YYEMPTY)                                        \
+      {                                                           \
+        yychar = (Token);                                         \
+        yylval = (Value);                                         \
+        YYPOPSTACK (yylen);                                       \
+        yystate = *yyssp;                                         \
+        goto yybackup;                                            \
+      }                                                           \
+    else                                                          \
+      {                                                           \
+        yyerror (YY_("syntax error: cannot back up")); \
+        YYERROR;                                                  \
+      }                                                           \
+  while (0)
 
 /* Error token number */
 #define YYTERROR        1
@@ -867,37 +863,37 @@ do {                                                                      \
 } while (0)
 
 
-/*----------------------------------------.
-| Print this symbol's value on YYOUTPUT.  |
-`----------------------------------------*/
+/*-----------------------------------.
+| Print this symbol's value on YYO.  |
+`-----------------------------------*/
 
 static void
-yy_symbol_value_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep)
+yy_symbol_value_print (FILE *yyo, int yytype, YYSTYPE const * const yyvaluep)
 {
-  FILE *yyo = yyoutput;
-  YYUSE (yyo);
+  FILE *yyoutput = yyo;
+  YYUSE (yyoutput);
   if (!yyvaluep)
     return;
 # ifdef YYPRINT
   if (yytype < YYNTOKENS)
-    YYPRINT (yyoutput, yytoknum[yytype], *yyvaluep);
+    YYPRINT (yyo, yytoknum[yytype], *yyvaluep);
 # endif
   YYUSE (yytype);
 }
 
 
-/*--------------------------------.
-| Print this symbol on YYOUTPUT.  |
-`--------------------------------*/
+/*---------------------------.
+| Print this symbol on YYO.  |
+`---------------------------*/
 
 static void
-yy_symbol_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep)
+yy_symbol_print (FILE *yyo, int yytype, YYSTYPE const * const yyvaluep)
 {
-  YYFPRINTF (yyoutput, "%s %s (",
+  YYFPRINTF (yyo, "%s %s (",
              yytype < YYNTOKENS ? "token" : "nterm", yytname[yytype]);
 
-  yy_symbol_value_print (yyoutput, yytype, yyvaluep);
-  YYFPRINTF (yyoutput, ")");
+  yy_symbol_value_print (yyo, yytype, yyvaluep);
+  YYFPRINTF (yyo, ")");
 }
 
 /*------------------------------------------------------------------.
@@ -931,7 +927,7 @@ do {                                                            \
 static void
 yy_reduce_print (yytype_int16 *yyssp, YYSTYPE *yyvsp, int yyrule)
 {
-  unsigned long int yylno = yyrline[yyrule];
+  unsigned long yylno = yyrline[yyrule];
   int yynrhs = yyr2[yyrule];
   int yyi;
   YYFPRINTF (stderr, "Reducing stack by rule %d (line %lu):\n",
@@ -942,7 +938,7 @@ yy_reduce_print (yytype_int16 *yyssp, YYSTYPE *yyvsp, int yyrule)
       YYFPRINTF (stderr, "   $%d = ", yyi + 1);
       yy_symbol_print (stderr,
                        yystos[yyssp[yyi + 1 - yynrhs]],
-                       &(yyvsp[(yyi + 1) - (yynrhs)])
+                       &yyvsp[(yyi + 1) - (yynrhs)]
                                               );
       YYFPRINTF (stderr, "\n");
     }
@@ -1046,7 +1042,10 @@ yytnamerr (char *yyres, const char *yystr)
           case '\\':
             if (*++yyp != '\\')
               goto do_not_strip_quotes;
-            /* Fall through.  */
+            else
+              goto append;
+
+          append:
           default:
             if (yyres)
               yyres[yyn] = *yyp;
@@ -1064,7 +1063,7 @@ yytnamerr (char *yyres, const char *yystr)
   if (! yyres)
     return yystrlen (yystr);
 
-  return yystpcpy (yyres, yystr) - yyres;
+  return (YYSIZE_T) (yystpcpy (yyres, yystr) - yyres);
 }
 # endif
 
@@ -1142,10 +1141,10 @@ yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
                 yyarg[yycount++] = yytname[yyx];
                 {
                   YYSIZE_T yysize1 = yysize + yytnamerr (YY_NULLPTR, yytname[yyx]);
-                  if (! (yysize <= yysize1
-                         && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
+                  if (yysize <= yysize1 && yysize1 <= YYSTACK_ALLOC_MAXIMUM)
+                    yysize = yysize1;
+                  else
                     return 2;
-                  yysize = yysize1;
                 }
               }
         }
@@ -1169,9 +1168,10 @@ yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
 
   {
     YYSIZE_T yysize1 = yysize + yystrlen (yyformat);
-    if (! (yysize <= yysize1 && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
+    if (yysize <= yysize1 && yysize1 <= YYSTACK_ALLOC_MAXIMUM)
+      yysize = yysize1;
+    else
       return 2;
-    yysize = yysize1;
   }
 
   if (*yymsg_alloc < yysize)
@@ -1297,23 +1297,31 @@ yyparse (void)
   yychar = YYEMPTY; /* Cause a token to be read.  */
   goto yysetstate;
 
+
 /*------------------------------------------------------------.
-| yynewstate -- Push a new state, which is found in yystate.  |
+| yynewstate -- push a new state, which is found in yystate.  |
 `------------------------------------------------------------*/
- yynewstate:
+yynewstate:
   /* In all cases, when you get here, the value and location stacks
      have just been pushed.  So pushing a state here evens the stacks.  */
   yyssp++;
 
- yysetstate:
-  *yyssp = yystate;
+
+/*--------------------------------------------------------------------.
+| yynewstate -- set current state (the top of the stack) to yystate.  |
+`--------------------------------------------------------------------*/
+yysetstate:
+  *yyssp = (yytype_int16) yystate;
 
   if (yyss + yystacksize - 1 <= yyssp)
+#if !defined yyoverflow && !defined YYSTACK_RELOCATE
+    goto yyexhaustedlab;
+#else
     {
       /* Get the current used size of the three stacks, in elements.  */
-      YYSIZE_T yysize = yyssp - yyss + 1;
+      YYSIZE_T yysize = (YYSIZE_T) (yyssp - yyss + 1);
 
-#ifdef yyoverflow
+# if defined yyoverflow
       {
         /* Give user a chance to reallocate the stack.  Use copies of
            these so that the &'s don't force the real ones into
@@ -1329,14 +1337,10 @@ yyparse (void)
                     &yyss1, yysize * sizeof (*yyssp),
                     &yyvs1, yysize * sizeof (*yyvsp),
                     &yystacksize);
-
         yyss = yyss1;
         yyvs = yyvs1;
       }
-#else /* no yyoverflow */
-# ifndef YYSTACK_RELOCATE
-      goto yyexhaustedlab;
-# else
+# else /* defined YYSTACK_RELOCATE */
       /* Extend the stack our own way.  */
       if (YYMAXDEPTH <= yystacksize)
         goto yyexhaustedlab;
@@ -1352,22 +1356,22 @@ yyparse (void)
           goto yyexhaustedlab;
         YYSTACK_RELOCATE (yyss_alloc, yyss);
         YYSTACK_RELOCATE (yyvs_alloc, yyvs);
-#  undef YYSTACK_RELOCATE
+# undef YYSTACK_RELOCATE
         if (yyss1 != yyssa)
           YYSTACK_FREE (yyss1);
       }
 # endif
-#endif /* no yyoverflow */
 
       yyssp = yyss + yysize - 1;
       yyvsp = yyvs + yysize - 1;
 
       YYDPRINTF ((stderr, "Stack size increased to %lu\n",
-                  (unsigned long int) yystacksize));
+                  (unsigned long) yystacksize));
 
       if (yyss + yystacksize - 1 <= yyssp)
         YYABORT;
     }
+#endif /* !defined yyoverflow && !defined YYSTACK_RELOCATE */
 
   YYDPRINTF ((stderr, "Entering state %d\n", yystate));
 
@@ -1376,11 +1380,11 @@ yyparse (void)
 
   goto yybackup;
 
+
 /*-----------.
 | yybackup.  |
 `-----------*/
 yybackup:
-
   /* Do appropriate processing given the current state.  Read a
      lookahead token if we need one and don't already have one.  */
 
@@ -1453,7 +1457,7 @@ yydefault:
 
 
 /*-----------------------------.
-| yyreduce -- Do a reduction.  |
+| yyreduce -- do a reduction.  |
 `-----------------------------*/
 yyreduce:
   /* yyn is the number of a rule to reduce with.  */
@@ -1474,7 +1478,7 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 247 "i386_parse.y" /* yacc.c:1648  */
+#line 247 "i386_parse.y" /* yacc.c:1652  */
     {
 		      if (error_message_count != 0)
 			error (EXIT_FAILURE, 0,
@@ -1482,29 +1486,29 @@ yyreduce:
 
 		      instrtable_out ();
 		    }
-#line 1486 "i386_parse.c" /* yacc.c:1648  */
+#line 1490 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 5:
-#line 261 "i386_parse.y" /* yacc.c:1648  */
+#line 261 "i386_parse.y" /* yacc.c:1652  */
     { new_bitfield ((yyvsp[-1].str), (yyvsp[0].num)); }
-#line 1492 "i386_parse.c" /* yacc.c:1648  */
+#line 1496 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 6:
-#line 263 "i386_parse.y" /* yacc.c:1648  */
+#line 263 "i386_parse.y" /* yacc.c:1652  */
     { new_bitfield ((yyvsp[0].str), -1); }
-#line 1498 "i386_parse.c" /* yacc.c:1648  */
+#line 1502 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 7:
-#line 265 "i386_parse.y" /* yacc.c:1648  */
+#line 265 "i386_parse.y" /* yacc.c:1652  */
     { new_bitfield ((yyvsp[0].str), -2); }
-#line 1504 "i386_parse.c" /* yacc.c:1648  */
+#line 1508 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 8:
-#line 267 "i386_parse.y" /* yacc.c:1648  */
+#line 267 "i386_parse.y" /* yacc.c:1652  */
     {
 		      struct synonym *newp = xmalloc (sizeof (*newp));
 		      newp->from = (yyvsp[-1].str);
@@ -1516,11 +1520,11 @@ yyreduce:
 		      else if (tsearch ( newp, &synonyms, compare_syn) == NULL)
 			error (EXIT_FAILURE, 0, "tsearch");
 		    }
-#line 1520 "i386_parse.c" /* yacc.c:1648  */
+#line 1524 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 12:
-#line 286 "i386_parse.y" /* yacc.c:1648  */
+#line 286 "i386_parse.y" /* yacc.c:1652  */
     {
 		      if ((yyvsp[-3].field) != NULL && strcmp ((yyvsp[-3].field)->name, "RE") != 0
 			  && strcmp ((yyvsp[-3].field)->name, "R") != 0)
@@ -1603,11 +1607,11 @@ yyreduce:
 			  ++ninstructions;
 			}
 		    }
-#line 1607 "i386_parse.c" /* yacc.c:1648  */
+#line 1611 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 14:
-#line 372 "i386_parse.y" /* yacc.c:1648  */
+#line 372 "i386_parse.y" /* yacc.c:1652  */
     {
 		      struct known_bitfield search;
 		      search.name = (yyvsp[0].str);
@@ -1622,17 +1626,17 @@ yyreduce:
 		      else
 			(yyval.field) = *res;
 		    }
-#line 1626 "i386_parse.c" /* yacc.c:1648  */
+#line 1630 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 15:
-#line 387 "i386_parse.y" /* yacc.c:1648  */
+#line 387 "i386_parse.y" /* yacc.c:1652  */
     { (yyval.field) = NULL; }
-#line 1632 "i386_parse.c" /* yacc.c:1648  */
+#line 1636 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 16:
-#line 391 "i386_parse.y" /* yacc.c:1648  */
+#line 391 "i386_parse.y" /* yacc.c:1652  */
     {
 		      check_bits ((yyvsp[0].bit));
 
@@ -1642,20 +1646,20 @@ yyreduce:
 		      runp->next = (yyvsp[0].bit);
 		      (yyval.bit) = (yyvsp[-2].bit);
 		    }
-#line 1646 "i386_parse.c" /* yacc.c:1648  */
+#line 1650 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 17:
-#line 401 "i386_parse.y" /* yacc.c:1648  */
+#line 401 "i386_parse.y" /* yacc.c:1652  */
     {
 		      check_bits ((yyvsp[0].bit));
 		      (yyval.bit) = (yyvsp[0].bit);
 		    }
-#line 1655 "i386_parse.c" /* yacc.c:1648  */
+#line 1659 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 18:
-#line 408 "i386_parse.y" /* yacc.c:1648  */
+#line 408 "i386_parse.y" /* yacc.c:1652  */
     {
 		      struct bitvalue *runp = (yyvsp[-1].bit);
 		      while (runp->next != NULL)
@@ -1663,39 +1667,39 @@ yyreduce:
 		      runp->next = (yyvsp[0].bit);
 		      (yyval.bit) = (yyvsp[-1].bit);
 		    }
-#line 1667 "i386_parse.c" /* yacc.c:1648  */
+#line 1671 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 19:
-#line 416 "i386_parse.y" /* yacc.c:1648  */
+#line 416 "i386_parse.y" /* yacc.c:1652  */
     { (yyval.bit) = (yyvsp[0].bit); }
-#line 1673 "i386_parse.c" /* yacc.c:1648  */
+#line 1677 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 20:
-#line 420 "i386_parse.y" /* yacc.c:1648  */
+#line 420 "i386_parse.y" /* yacc.c:1652  */
     {
 		      (yyval.bit) = xmalloc (sizeof (struct bitvalue));
 		      (yyval.bit)->type = zeroone;
 		      (yyval.bit)->value = 0;
 		      (yyval.bit)->next = NULL;
 		    }
-#line 1684 "i386_parse.c" /* yacc.c:1648  */
+#line 1688 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 21:
-#line 427 "i386_parse.y" /* yacc.c:1648  */
+#line 427 "i386_parse.y" /* yacc.c:1652  */
     {
 		      (yyval.bit) = xmalloc (sizeof (struct bitvalue));
 		      (yyval.bit)->type = zeroone;
 		      (yyval.bit)->value = 1;
 		      (yyval.bit)->next = NULL;
 		    }
-#line 1695 "i386_parse.c" /* yacc.c:1648  */
+#line 1699 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 22:
-#line 434 "i386_parse.y" /* yacc.c:1648  */
+#line 434 "i386_parse.y" /* yacc.c:1652  */
     {
 		      (yyval.bit) = xmalloc (sizeof (struct bitvalue));
 		      struct known_bitfield search;
@@ -1715,23 +1719,23 @@ yyreduce:
 			}
 		      (yyval.bit)->next = NULL;
 		    }
-#line 1719 "i386_parse.c" /* yacc.c:1648  */
+#line 1723 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 23:
-#line 456 "i386_parse.y" /* yacc.c:1648  */
+#line 456 "i386_parse.y" /* yacc.c:1652  */
     { (yyval.arg) = (yyvsp[0].arg); }
-#line 1725 "i386_parse.c" /* yacc.c:1648  */
+#line 1729 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 24:
-#line 458 "i386_parse.y" /* yacc.c:1648  */
+#line 458 "i386_parse.y" /* yacc.c:1652  */
     { (yyval.arg) = NULL; }
-#line 1731 "i386_parse.c" /* yacc.c:1648  */
+#line 1735 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 25:
-#line 462 "i386_parse.y" /* yacc.c:1648  */
+#line 462 "i386_parse.y" /* yacc.c:1652  */
     {
 		      struct argument *runp = (yyvsp[-2].arg);
 		      while (runp->next != NULL)
@@ -1741,21 +1745,21 @@ yyreduce:
 		      runp->next->next = NULL;
 		      (yyval.arg) = (yyvsp[-2].arg);
 		    }
-#line 1745 "i386_parse.c" /* yacc.c:1648  */
+#line 1749 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 26:
-#line 472 "i386_parse.y" /* yacc.c:1648  */
+#line 472 "i386_parse.y" /* yacc.c:1652  */
     {
 		      (yyval.arg) = xmalloc (sizeof (struct argument));
 		      (yyval.arg)->name = combine ((yyvsp[0].name));
 		      (yyval.arg)->next = NULL;
 		    }
-#line 1755 "i386_parse.c" /* yacc.c:1648  */
+#line 1759 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 27:
-#line 480 "i386_parse.y" /* yacc.c:1648  */
+#line 480 "i386_parse.y" /* yacc.c:1652  */
     {
 		      struct argname *runp = (yyvsp[-1].name);
 		      while (runp->next != NULL)
@@ -1763,17 +1767,17 @@ yyreduce:
 		      runp->next = (yyvsp[0].name);
 		      (yyval.name) = (yyvsp[-1].name);
 		    }
-#line 1767 "i386_parse.c" /* yacc.c:1648  */
+#line 1771 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 28:
-#line 488 "i386_parse.y" /* yacc.c:1648  */
+#line 488 "i386_parse.y" /* yacc.c:1652  */
     { (yyval.name) = (yyvsp[0].name); }
-#line 1773 "i386_parse.c" /* yacc.c:1648  */
+#line 1777 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 29:
-#line 491 "i386_parse.y" /* yacc.c:1648  */
+#line 491 "i386_parse.y" /* yacc.c:1652  */
     {
 		      (yyval.name) = xmalloc (sizeof (struct argname));
 		      (yyval.name)->type = nfield;
@@ -1805,11 +1809,11 @@ yyreduce:
 		      else
 			(yyval.name)->field = *res;
 		    }
-#line 1809 "i386_parse.c" /* yacc.c:1648  */
+#line 1813 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 30:
-#line 523 "i386_parse.y" /* yacc.c:1648  */
+#line 523 "i386_parse.y" /* yacc.c:1652  */
     {
 		      (yyval.name) = xmalloc (sizeof (struct argname));
 		      (yyval.name)->type = string;
@@ -1818,22 +1822,22 @@ yyreduce:
 		      (yyval.name)->str[0] = (yyvsp[0].ch);
 		      (yyval.name)->str[1] = '\0';
 		    }
-#line 1822 "i386_parse.c" /* yacc.c:1648  */
+#line 1826 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 31:
-#line 532 "i386_parse.y" /* yacc.c:1648  */
+#line 532 "i386_parse.y" /* yacc.c:1652  */
     {
 		      (yyval.name) = xmalloc (sizeof (struct argname));
 		      (yyval.name)->type = string;
 		      (yyval.name)->next = NULL;
 		      (yyval.name)->str = (yyvsp[0].str);
 		    }
-#line 1833 "i386_parse.c" /* yacc.c:1648  */
+#line 1837 "i386_parse.c" /* yacc.c:1652  */
     break;
 
   case 32:
-#line 539 "i386_parse.y" /* yacc.c:1648  */
+#line 539 "i386_parse.y" /* yacc.c:1652  */
     {
 		      (yyval.name) = xmalloc (sizeof (struct argname));
 		      (yyval.name)->type = string;
@@ -1842,11 +1846,11 @@ yyreduce:
 		      (yyval.name)->str[0] = ':';
 		      (yyval.name)->str[1] = '\0';
 		    }
-#line 1846 "i386_parse.c" /* yacc.c:1648  */
+#line 1850 "i386_parse.c" /* yacc.c:1652  */
     break;
 
 
-#line 1850 "i386_parse.c" /* yacc.c:1648  */
+#line 1854 "i386_parse.c" /* yacc.c:1652  */
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
@@ -1871,14 +1875,13 @@ yyreduce:
   /* Now 'shift' the result of the reduction.  Determine what state
      that goes to, based on the state we popped back to and the rule
      number reduced by.  */
-
-  yyn = yyr1[yyn];
-
-  yystate = yypgoto[yyn - YYNTOKENS] + *yyssp;
-  if (0 <= yystate && yystate <= YYLAST && yycheck[yystate] == *yyssp)
-    yystate = yytable[yystate];
-  else
-    yystate = yydefgoto[yyn - YYNTOKENS];
+  {
+    const int yylhs = yyr1[yyn] - YYNTOKENS;
+    const int yyi = yypgoto[yylhs] + *yyssp;
+    yystate = (0 <= yyi && yyi <= YYLAST && yycheck[yyi] == *yyssp
+               ? yytable[yyi]
+               : yydefgoto[yylhs]);
+  }
 
   goto yynewstate;
 
@@ -1961,12 +1964,10 @@ yyerrlab:
 | yyerrorlab -- error raised explicitly by YYERROR.  |
 `---------------------------------------------------*/
 yyerrorlab:
-
-  /* Pacify compilers like GCC when the user code never invokes
-     YYERROR and the label yyerrorlab therefore never appears in user
-     code.  */
-  if (/*CONSTCOND*/ 0)
-     goto yyerrorlab;
+  /* Pacify compilers when the user code never invokes YYERROR and the
+     label yyerrorlab therefore never appears in user code.  */
+  if (0)
+    YYERROR;
 
   /* Do not reclaim the symbols of the rule whose action triggered
      this YYERROR.  */
@@ -2028,6 +2029,7 @@ yyacceptlab:
   yyresult = 0;
   goto yyreturn;
 
+
 /*-----------------------------------.
 | yyabortlab -- YYABORT comes here.  |
 `-----------------------------------*/
@@ -2035,6 +2037,7 @@ yyabortlab:
   yyresult = 1;
   goto yyreturn;
 
+
 #if !defined yyoverflow || YYERROR_VERBOSE
 /*-------------------------------------------------.
 | yyexhaustedlab -- memory exhaustion comes here.  |
@@ -2045,6 +2048,10 @@ yyexhaustedlab:
   /* Fall through.  */
 #endif
 
+
+/*-----------------------------------------------------.
+| yyreturn -- parsing is finished, return the result.  |
+`-----------------------------------------------------*/
 yyreturn:
   if (yychar != YYEMPTY)
     {
@@ -2074,7 +2081,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 549 "i386_parse.y" /* yacc.c:1907  */
+#line 549 "i386_parse.y" /* yacc.c:1918  */
 
 
 static void
