@@ -26,6 +26,10 @@
    the GNU Lesser General Public License along with this program.  If
    not, see <http://www.gnu.org/licenses/>.  */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
 #include "libdwflP.h"
 #include "system.h"
 
@@ -135,14 +139,14 @@ open_stream (int fd, off_t start_offset, struct unzip_state *state)
 {
     int d = dup (fd);
     if (unlikely (d < 0))
-      return DWFL_E_BADELF;
+      return DWFL_E_ERRNO;
     if (start_offset != 0)
       {
 	off_t off = lseek (d, start_offset, SEEK_SET);
 	if (off != start_offset)
 	  {
 	    close (d);
-	    return DWFL_E_BADELF;
+	    return DWFL_E_ERRNO;
 	  }
       }
     state->zf = gzdopen (d, "r");
@@ -284,6 +288,7 @@ unzip (int fd, off_t start_offset,
   if (result == DWFL_E_NOERROR && gzdirect (state.zf))
     {
       gzclose (state.zf);
+      /* Not a compressed stream after all.  */
       return fail (&state, DWFL_E_BADELF);
     }
 
