@@ -36,11 +36,7 @@
 
 
 const char *
-ebl_symbol_binding_name (ebl, binding, buf, len)
-     Ebl *ebl;
-     int binding;
-     char *buf;
-     size_t len;
+ebl_symbol_binding_name (Ebl *ebl, int binding, char *buf, size_t len)
 {
   const char *res;
 
@@ -62,6 +58,7 @@ ebl_symbol_binding_name (ebl, binding, buf, len)
 	  if (binding >= STB_LOPROC && binding <= STB_HIPROC)
 	    snprintf (buf, len, "LOPROC+%d", binding - STB_LOPROC);
 	  else if (binding == STB_GNU_UNIQUE
+		   && ebl != NULL
 		   && (ident = elf_getident (ebl->elf, NULL)) != NULL
 		   && ident[EI_OSABI] == ELFOSABI_LINUX)
 	    return "GNU_UNIQUE";
