@@ -30,13 +30,7 @@
 /* This is the interface for the Elfutils Backend Library.
    It is a completely UNSUPPORTED interface.  Don't use any libebl
    function directly.  These are only for internal elfutils backends
-   and tools.  There is NO source or binary compatible guarantee.
-
-   The ABI of the backend modules is not guaranteed.  Really, no guarantee
-   whatsoever.  We are enforcing this in the code.  The modules and their
-   users must match.  No third-party EBL module are supported or allowed.
-   The only reason there are separate modules is to not have the code for
-   all architectures in all the binaries.  */
+   and tools.  There is NO source or binary compatible guarantee.  */
 
 
 #ifndef _LIBEBL_H
