@@ -185,7 +185,7 @@ frame_callback (Dwfl_Frame *state, void *frame_arg)
     symname = dwfl_module_addrname (mod, pc_adjusted);
 
   printf ("#%2d %#" PRIx64 "%4s\t%s\n", *framenop, (uint64_t) pc,
-	  ! isactivation ? "- 1" : "", symname);
+	  ! isactivation ? "- 1" : "", symname ?: "<null>");
   pid_t tid = dwfl_thread_tid (thread);
   callback_verify (tid, *framenop, pc, symname, dwfl);
   (*framenop)++;
@@ -296,7 +296,6 @@ prepare_thread (pid_t pid2 __attribute__ ((unused)),
 
 #include <asm/unistd.h>
 #include <unistd.h>
-#define tgkill(pid, tid, sig) syscall (__NR_tgkill, (pid), (tid), (sig))
 
 static void
 report_pid (Dwfl *dwfl, pid_t pid)
